/*
 * pwm_dma.c
 *
 *  Created on: Jun 20, 2022
 *      Author: hussamaldean
 */

#include <WS2812.h>
#include "stm32f4xx.h"
#include "math.h"
#include "string.h"


volatile uint8_t finished;

#define  Num_of_LEDs 24

void  WS2812_init(void){
#define CH6   ((0x06<<25))
#define AF01   (0x01)
//GPIO init
RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN;
GPIOA->MODER|=GPIO_MODER_MODE8_1;
GPIOA->MODER&=~GPIO_MODER_MODE8_0;
GPIOA->AFR[1]|=(AF01<<0);
//init timer;
RCC->APB2ENR|=RCC_APB2ENR_TIM1EN; //enable clock access to tim1
TIM1->PSC=0; //set prescaller to 0 (no divider)
TIM1->ARR=90-1; //set the maximum count value
TIM1->CNT=0; //reset the current count
TIM1->DIER|=TIM_DIER_CC1DE;
TIM1->CCMR1|=TIM_CCMR1_OC1M_2|TIM_CCMR1_OC1M_1|TIM_CCMR1_OC1PE; //configure the pin as PWM
TIM1->CCER|=TIM_CCER_CC1E; //enable channel1
TIM1->DMAR|=0x01;
TIM1->BDTR=TIM_BDTR_MOE;

//init dma
RCC->AHB1ENR|=RCC_AHB1ENR_DMA2EN;
DMA2_Stream1->CR&=~DMA_SxCR_EN;
while((DMA2_Stream1->CR)&DMA_SxCR_EN){;}
DMA2_Stream1->CR|=CH6|DMA_SxCR_MSIZE_0|DMA_SxCR_PSIZE_0|DMA_SxCR_MINC|DMA_SxCR_DIR_0|DMA_SxCR_TCIE;
DMA2_Stream1->PAR=(uint32_t)(&TIM1->CCR1);
NVIC_EnableIRQ(DMA2_Stream1_IRQn);
}

uint8_t LED_Data[Num_of_LEDs][4];
uint8_t LED_Mod[Num_of_LEDs][4];  // for brightness


void Set_LED (int LEDnum, int Red, int Green, int Blue)
{
	LED_Data[LEDnum][0] = LEDnum;
	LED_Data[LEDnum][1] = Green;
	LED_Data[LEDnum][2] = Red;
	LED_Data[LEDnum][3] = Blue;
}





void Set_Brightness (int brightness)  // 0-45
{
#define PI 3.14159265


	if (brightness > 45) brightness = 45;
	for (int i=0; i<Num_of_LEDs; i++)
	{
		LED_Mod[i][0] = LED_Data[i][0];
		for (int j=1; j<4; j++)
		{
			float angle = 90-brightness;  // in degrees
			angle = angle*PI / 180;  // in rad
			LED_Mod[i][j] = (LED_Data[i][j])/(tan(angle));
		}
	}

}


uint16_t pwmData[(24*Num_of_LEDs)+50];

void WS2812_display()
{
	uint32_t indx=0;
	uint32_t color;
	for (int i= 0; i<Num_of_LEDs; i++)
		{
			color = ((LED_Mod[i][1]<<16) | (LED_Mod[i][2]<<8) | (LED_Mod[i][3]));

			for (int i=23; i>=0; i--)
			{
				if (color&(1<<i))
				{
					pwmData[indx] = 60;  // 2/3 of 90
				}

				else pwmData[indx] = 30;  // 1/3 of 90

				indx++;
			}

		}

	for (int i=0; i<50; i++)
		{
			pwmData[indx] = 0;
			indx++;
		}


	DMA2_Stream1->M0AR=(uint32_t)(&pwmData);
	DMA2_Stream1->NDTR=indx;
	DMA2->LIFCR|=DMA_LIFCR_CFEIF1|DMA_LIFCR_CDMEIF1|DMA_LIFCR_CTEIF1|DMA_LIFCR_CHTIF1|DMA_LIFCR_CTCIF1;
	TIM1->CR1|=TIM_CR1_CEN; //enable timer
	DMA2_Stream1->CR|=DMA_SxCR_EN;
	while(finished==0);
	finished=0;
	}


void DMA2_Stream1_IRQHandler(void)
	{
	if((DMA2->LISR&DMA_LISR_TCIF1))
		{
		finished=1;
		TIM1->CR1&=~TIM_CR1_CEN;
		DMA2->LIFCR|=DMA_LIFCR_CTCIF1;

		}

	}
